==> tp_client-0.8.1-py3-none-any.whl: tp_client/client.py <==
import logging
from typing import Any, Dict, List, Optional, Tuple, Type, TypeVar, Union
from urllib.parse import urljoin, urlparse

import requests
from requests.models import Response

from tp_client.models import (
    BaseTimetable,
    BookingsTimetable,
    CourseTimetable,
    EvuTimetable,
    ProgramTimetable,
    YeargroupTimetable,
    ClassTimetable,
    EvuTimetable,
    RoomData,
    Rooms,
    StaffTimetable,
    StudentTimetable,
)

logger = logging.getLogger(__name__)

T = TypeVar("T")

DEFAULT_USE_MODELS = True


class TpEndpoints:
    """Get endpoints from an API base URL."""

    def __init__(self, url: str):
        self.base_url = url

    def __repr__(self):
        return "{cls.__name__}({url!r})".format(cls=type(self), url=self.base_url)

    def add_base(self, path: str):
        return urljoin(self.base_url, path)

    def get_timetable_for_staff(self):
        # https://tp-test.uio.no/ws/1.4/staff.php?id=olanord&fromdate=2017-01-10&todate=2017-08-20&lang=no
        return self.add_base("ws/1.4/staff.php")

    def get_timetable_for_course(self):
        # https://tp-test.uio.no/ws/1.4/course.php?id=IN1000&fromdate=2018-07-01&todate=2019-12-31&lang=no
        return self.add_base("ws/1.4/course.php")

    def get_timetable_for_evu(self):
        # https://gw-uio.intark.uh-it.no/tptest-uio/ws/1.4/evu.php?id=11-RESA2702V-2021H-2022V&sem=22h
        return self.add_base("ws/1.4/evu.php")

    def get_room_data(self):
        # https://tp-test.uio.no/ws/room/2.0/?id=BL05109
        return self.add_base("ws/room/2.0/")

    def get_all_rooms(self):
        # https://tp-test.uio.no/ws/room/2.0/allrooms.php
        return self.add_base("ws/room/2.0/allrooms.php")

    def get_timetable_for_single_student(self):
        return self.add_base("ws/1.4/single_student.php")

    def get_timetable_for_personal_bookings(self):
        return self.add_base("ws/1.4/bookings.php")

    def get_timetable_for_program_yeargroup_class(self):
        return self.add_base("ws/1.4/program.php")


class TpClient:
    def __init__(
        self,
        url: str,
        headers: Optional[Dict[str, str]] = None,
        rewrite_url: Optional[Tuple[str, str]] = None,
        return_objects: bool = True,
        use_sessions: bool = True,
    ):
        """
        TP API client.

        :param str base_url: Base API URL
        :param dict headers: Append extra headers to all requests
        :param tuple(str, str) rewrite_url: Rewrite URLs before sending requests (from, to)
        :param bool return_objects: Return verified objects
        :param bool use_sessions: Keep-alive HTTP connections
        """
        self.urls = TpEndpoints(url)
        self.headers: Dict[str, str] = {} if headers is None else headers
        self.rewrite_url = rewrite_url
        self.return_objects = return_objects

        if use_sessions:
            self.session = requests.Session()
        else:
            self.session = requests

    def _build_request_headers(self, headers: Optional[Dict[str, str]]):
        request_headers: Dict[str, str] = {}
        for h in self.headers:
            request_headers[h] = self.headers[h]
        for h in headers or ():
            request_headers[h] = headers[h]
        return request_headers

    def call(
        self,
        method_name: str,
        url: str,
        headers: Optional[Dict[str, str]] = None,
        params: Optional[Dict[str, Any]] = None,
        return_response: bool = False,
        **kwargs: Any,
    ) -> Union[Dict[str, Any], Response]:
        headers = self._build_request_headers(headers)
        if params is None:
            params = {}
        if self.rewrite_url is not None:
            url = url.replace(*self.rewrite_url)
        logger.debug(
            "Calling %s %s with params=%r", method_name, urlparse(url).path, params
        )
        r = self.session.request(
            method_name, url, headers=headers, params=params, **kwargs
        )
        if r.status_code in (500, 400, 401):
            logger.warn("Got HTTP %d: %r", r.status_code, r.content)
        if return_response:
            return r
        r.raise_for_status()
        return r.json()

    def get(self, url: str, **kwargs: Any):
        return self.call("GET", url, **kwargs)

    def post(self, url: str, **kwargs: Any):
        return self.call("POST", url, **kwargs)

    def put(self, url: str, **kwargs: Any):
        return self.call("PUT", url, **kwargs)

    @staticmethod
    def _verify_interval_or_semester(
        start: Optional[str], end: Optional[str], semester: Optional[str]
    ) -> None:
        assert ((start and end) and not semester) or (semester and not (start and end))

    def maybe_object(
        self, cls: Type[T], obj: Dict[str, Any]
    ) -> Union[T, Dict[Any, Any]]:
        if not self.return_objects:
            return obj
        return cls.from_dict(obj)

    def get_timetable_for_staff(
        self,
        uid: str,
        start: Optional[str] = None,
        end: Optional[str] = None,
        semester: Optional[str] = None,
        split_intervals: Optional[bool] = None,
        lang: str = "no",
    ) -> Union[StaffTimetable, Dict[str, Any]]:
        """
        Get the timetable for a staff member.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str uid: Staff username
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param bool split_intervals: Set the split intervals parameter
        :param str lang: Language
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_staff()
        params = {
            "id": uid,
            "fromdate": start,
            "todate": end,
            "sem": semester,
            "lang": lang,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        timetable: Dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(StaffTimetable, timetable)

    def get_timetable_for_course(
        self,
        course_id: str,
        start: Optional[str] = None,
        end: Optional[str] = None,
        semester: Optional[str] = None,
        lang: str = "no",
        split_intervals: Optional[bool] = None,
        exam: Optional[bool] = None,
    ) -> Union[CourseTimetable, Dict[str, Any]]:
        """
        Get the timetable for a course for a given date interval or semester.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str course_id: Course ID
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param str lang: Language
        :param bool split_intervals: Set the split intervals parameter
        :param bool exam: Include exams in result
        """
        # TODO: this endpoint accepts a "termnr". what does it do?
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_course()
        params = {
            "id": course_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
            "lang": lang,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        if exam is not None:
            params["exam"] = "true" if exam else "false"
        timetable: Dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(CourseTimetable, timetable)

    def get_timetable_for_evu(
        self,
        evu_id: str,
        start: Optional[str] = None,
        end: Optional[str] = None,
        semester: Optional[str] = None,
        split_intervals: Optional[bool] = None,
    ) -> Union[EvuTimetable, Dict[str, Any]]:
        """
        Get the timetable for a EVU course for a given date interval or semester.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str course_id: Course ID
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param bool split_intervals: Set the split intervals parameter
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_evu()
        params = {
            "id": evu_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"

        timetable: Dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(EvuTimetable, timetable)

    def get_timetable_for_single_student(
        self,
        student_id: str,
        start: Optional[str] = None,
        end: Optional[str] = None,
        semester: Optional[str] = None,
        lang: str = "no",
        split_intervals: Optional[bool] = None,
    ) -> Union[StudentTimetable, Dict[str, Any]]:
        """
        Get the timetable for a student given date interval or semester.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str student_id: Student username
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param str lang: Language
        :param bool split_intervals: Set the split intervals parameter
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_single_student()
        params = {
            "id": student_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
            "lang": lang,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        timetable: Dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(StudentTimetable, timetable)

    def get_timetable_for_personal_bookings(
        self,
        uid: str,
        start: Optional[str] = None,
        end: Optional[str] = None,
        semester: Optional[str] = None,
    ) -> Union[BookingsTimetable, Dict[str, Any]]:
        """
        Get the timetable for a personal booking given date interval or semester.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str uid: Username of the person booking
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_personal_bookings()
        params = {
            "id": uid,
            "fromdate": start,
            "todate": end,
            "sem": semester,
        }
        timetable: Dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(BookingsTimetable, timetable)

    def get_timetable_for_program(
        self,
        program_id: Optional[List[str]] = None,
        start: Optional[str] = None,
        end: Optional[str] = None,
        semester: Optional[str] = None,
        split_intervals: Optional[bool] = None,
    ) -> Union[ProgramTimetable, Dict[str, Any]]:
        """
        Get the timetable for a study program.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str program_id: ID of study program (e.g MNB-BIOS)
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param bool split_intervals: Set the split intervals parameter
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_program_yeargroup_class()
        params = {
            "id[]": program_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        timetable: Dict[str, Any] = self.get(url, params=params)
        # we add the program ID to the timetable object
        timetable["requested_program_id"] = program_id
        return self.maybe_object(ProgramTimetable, timetable)

    def get_timetable_for_yeargroup(
        self,
        yeargroup_id: Optional[List[str]] = None,
        start: Optional[str] = None,
        end: Optional[str] = None,
        semester: Optional[str] = None,
        split_intervals: Optional[bool] = None,
    ) -> Union[YeargroupTimetable, Dict[str, Any]]:
        """
        Get the timetable for a yeargroup (kull).

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str yeargroup_id: ID of year group (e.g MNB-BIOS-2017-HØST)
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param bool split_intervals: Set the split intervals parameter
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_program_yeargroup_class()
        params = {
            "kullid[]": yeargroup_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        timetable: Dict[str, Any] = self.get(url, params=params)
        # we add the yeargroup ID to the timetable object
        timetable["requested_yeargroup_id"] = yeargroup_id
        return self.maybe_object(YeargroupTimetable, timetable)

    def get_timetable_for_class(
        self,
        class_id: Optional[List[str]] = None,
        start: Optional[str] = None,
        end: Optional[str] = None,
        semester: Optional[str] = None,
        split_intervals: Optional[bool] = None,
    ) -> Union[ClassTimetable, Dict[str, Any]]:
        """
        Get the timetable for a class (kullklasse).

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str class_id: ID of year group class (e.g MNB-BIOS-2017-HØST-H)
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param bool split_intervals: Set the split intervals parameter
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_program_yeargroup_class()
        params = {
            "classid[]": class_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        timetable: Dict[str, Any] = self.get(url, params=params)
        # we add the class ID to the timetable object
        timetable["requested_class_id"] = class_id
        return self.maybe_object(ClassTimetable, timetable)

    def get_room_data(self, room_id: str) -> Union[RoomData, Dict[Any, Any]]:
        """
        Get a room.

        :param str room_id: Room ID
        """
        url = self.urls.get_room_data()
        params = {
            "id": room_id,
        }
        room_data: Dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(RoomData, room_data)

    def get_all_rooms(self):
        """
        Get all active rooms.
        """
        url = self.urls.get_all_rooms()
        all_rooms: Dict[str, Any] = self.get(url)
        return self.maybe_object(Rooms, all_rooms)


def get_client(config_dict: Dict[str, Any]):
    """
    Get a TpClient from configuration.
    """
    return TpClient(
        url=config_dict["url"],
        headers=config_dict.get("headers"),
        rewrite_url=config_dict.get("rewrite_url"),
    )

==> tp_client-0.8.2-py3-none-any.whl: tp_client/client.py <==
import logging
from typing import Any, TypeVar
from urllib.parse import urljoin, urlparse

import requests
from requests.models import Response

from tp_client.models import (
    BookingsTimetable,
    ClassTimetable,
    CourseTimetable,
    EvuTimetable,
    ProgramTimetable,
    RoomData,
    Rooms,
    StaffTimetable,
    StudentTimetable,
    YeargroupTimetable,
)

logger = logging.getLogger(__name__)

T = TypeVar("T")

DEFAULT_USE_MODELS = True


class TpEndpoints:
    """Get endpoints from an API base URL."""

    def __init__(self, url: str):
        self.base_url = url

    def __repr__(self):
        return f"{type(self).__name__}({self.base_url!r})"

    def add_base(self, path: str):
        return urljoin(self.base_url, path)

    def get_timetable_for_staff(self):
        # https://tp-test.uio.no/ws/1.4/staff.php?id=olanord&fromdate=2017-01-10&todate=2017-08-20&lang=no
        return self.add_base("ws/1.4/staff.php")

    def get_timetable_for_course(self):
        # https://tp-test.uio.no/ws/1.4/course.php?id=IN1000&fromdate=2018-07-01&todate=2019-12-31&lang=no
        return self.add_base("ws/1.4/course.php")

    def get_timetable_for_evu(self):
        # https://gw-uio.intark.uh-it.no/tptest-uio/ws/1.4/evu.php?id=11-RESA2702V-2021H-2022V&sem=22h
        return self.add_base("ws/1.4/evu.php")

    def get_room_data(self):
        # https://tp-test.uio.no/ws/room/2.0/?id=BL05109
        return self.add_base("ws/room/2.0/")

    def get_all_rooms(self):
        # https://tp-test.uio.no/ws/room/2.0/allrooms.php
        return self.add_base("ws/room/2.0/allrooms.php")

    def get_timetable_for_single_student(self):
        return self.add_base("ws/1.4/single_student.php")

    def get_timetable_for_personal_bookings(self):
        return self.add_base("ws/1.4/bookings.php")

    def get_timetable_for_program_yeargroup_class(self):
        return self.add_base("ws/1.4/program.php")


class TpClient:
    def __init__(
        self,
        url: str,
        headers: dict[str, str] | None = None,
        rewrite_url: tuple[str, str] | None = None,
        return_objects: bool = True,
        use_sessions: bool = True,
    ):
        """
        TP API client.

        :param str base_url: Base API URL
        :param dict headers: Append extra headers to all requests
        :param tuple(str, str) rewrite_url: Rewrite URLs before sending requests (from, to)
        :param bool return_objects: Return verified objects
        :param bool use_sessions: Keep-alive HTTP connections
        """
        self.urls = TpEndpoints(url)
        self.headers: dict[str, str] = {} if headers is None else headers
        self.rewrite_url = rewrite_url
        self.return_objects = return_objects

        if use_sessions:
            self.session = requests.Session()
        else:
            self.session = requests

    def _build_request_headers(self, headers: dict[str, str] | None):
        request_headers: dict[str, str] = {}
        for h in self.headers:
            request_headers[h] = self.headers[h]
        for h in headers or ():
            request_headers[h] = headers[h]
        return request_headers

    def call(
        self,
        method_name: str,
        url: str,
        headers: dict[str, str] | None = None,
        params: dict[str, Any] | None = None,
        return_response: bool = False,
        **kwargs: Any,
    ) -> dict[str, Any] | Response:
        headers = self._build_request_headers(headers)
        if params is None:
            params = {}
        if self.rewrite_url is not None:
            url = url.replace(*self.rewrite_url)
        logger.debug(
            "Calling %s %s with params=%r", method_name, urlparse(url).path, params
        )
        r = self.session.request(
            method_name, url, headers=headers, params=params, **kwargs
        )
        if r.status_code in (500, 400, 401):
            logger.warn("Got HTTP %d: %r", r.status_code, r.content)
        if return_response:
            return r
        r.raise_for_status()
        return r.json()

    def get(self, url: str, **kwargs: Any):
        return self.call("GET", url, **kwargs)

    def post(self, url: str, **kwargs: Any):
        return self.call("POST", url, **kwargs)

    def put(self, url: str, **kwargs: Any):
        return self.call("PUT", url, **kwargs)

    @staticmethod
    def _verify_interval_or_semester(
        start: str | None, end: str | None, semester: str | None
    ) -> None:
        assert ((start and end) and not semester) or (semester and not (start and end))

    def maybe_object(self, cls: type[T], obj: dict[str, Any]) -> T | dict[Any, Any]:
        if not self.return_objects:
            return obj
        return cls.from_dict(obj)

    def get_timetable_for_staff(
        self,
        uid: str,
        start: str | None = None,
        end: str | None = None,
        semester: str | None = None,
        split_intervals: bool | None = None,
        lang: str = "no",
    ) -> StaffTimetable | dict[str, Any]:
        """
        Get the timetable for a staff member.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str uid: Staff username
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param bool split_intervals: Set the split intervals parameter
        :param str lang: Language
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_staff()
        params = {
            "id": uid,
            "fromdate": start,
            "todate": end,
            "sem": semester,
            "lang": lang,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        timetable: dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(StaffTimetable, timetable)

    def get_timetable_for_course(
        self,
        course_id: str,
        start: str | None = None,
        end: str | None = None,
        semester: str | None = None,
        lang: str = "no",
        split_intervals: bool | None = None,
        exam: bool | None = None,
    ) -> CourseTimetable | dict[str, Any]:
        """
        Get the timetable for a course for a given date interval or semester.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str course_id: Course ID
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param str lang: Language
        :param bool split_intervals: Set the split intervals parameter
        :param bool exam: Include exams in result
        """
        # TODO: this endpoint accepts a "termnr". what does it do?
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_course()
        params = {
            "id": course_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
            "lang": lang,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        if exam is not None:
            params["exam"] = "true" if exam else "false"
        timetable: dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(CourseTimetable, timetable)

    def get_timetable_for_evu(
        self,
        evu_id: str,
        start: str | None = None,
        end: str | None = None,
        semester: str | None = None,
        split_intervals: bool | None = None,
    ) -> EvuTimetable | dict[str, Any]:
        """
        Get the timetable for a EVU course for a given date interval or semester.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str course_id: Course ID
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param bool split_intervals: Set the split intervals parameter
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_evu()
        params = {
            "id": evu_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"

        timetable: dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(EvuTimetable, timetable)

    def get_timetable_for_single_student(
        self,
        student_id: str,
        start: str | None = None,
        end: str | None = None,
        semester: str | None = None,
        lang: str = "no",
        split_intervals: bool | None = None,
    ) -> StudentTimetable | dict[str, Any]:
        """
        Get the timetable for a student given date interval or semester.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str student_id: Student username
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param str lang: Language
        :param bool split_intervals: Set the split intervals parameter
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_single_student()
        params = {
            "id": student_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
            "lang": lang,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        timetable: dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(StudentTimetable, timetable)

    def get_timetable_for_personal_bookings(
        self,
        uid: str,
        start: str | None = None,
        end: str | None = None,
        semester: str | None = None,
    ) -> BookingsTimetable | dict[str, Any]:
        """
        Get the timetable for a personal booking given date interval or semester.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str uid: Username of the person booking
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_personal_bookings()
        params = {
            "id": uid,
            "fromdate": start,
            "todate": end,
            "sem": semester,
        }
        timetable: dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(BookingsTimetable, timetable)

    def get_timetable_for_program(
        self,
        program_id: list[str] | None = None,
        start: str | None = None,
        end: str | None = None,
        semester: str | None = None,
        split_intervals: bool | None = None,
    ) -> ProgramTimetable | dict[str, Any]:
        """
        Get the timetable for a study program.

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str program_id: ID of study program (e.g MNB-BIOS)
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param bool split_intervals: Set the split intervals parameter
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_program_yeargroup_class()
        params = {
            "id[]": program_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        timetable: dict[str, Any] = self.get(url, params=params)
        # we add the program ID to the timetable object
        timetable["requested_program_id"] = program_id
        return self.maybe_object(ProgramTimetable, timetable)

    def get_timetable_for_yeargroup(
        self,
        yeargroup_id: list[str] | None = None,
        start: str | None = None,
        end: str | None = None,
        semester: str | None = None,
        split_intervals: bool | None = None,
    ) -> YeargroupTimetable | dict[str, Any]:
        """
        Get the timetable for a yeargroup (kull).

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str yeargroup_id: ID of year group (e.g MNB-BIOS-2017-HØST)
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param bool split_intervals: Set the split intervals parameter
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_program_yeargroup_class()
        params = {
            "kullid[]": yeargroup_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        timetable: dict[str, Any] = self.get(url, params=params)
        # we add the yeargroup ID to the timetable object
        timetable["requested_yeargroup_id"] = yeargroup_id
        return self.maybe_object(YeargroupTimetable, timetable)

    def get_timetable_for_class(
        self,
        class_id: list[str] | None = None,
        start: str | None = None,
        end: str | None = None,
        semester: str | None = None,
        split_intervals: bool | None = None,
    ) -> ClassTimetable | dict[str, Any]:
        """
        Get the timetable for a class (kullklasse).

        Semester and date intervals are mutually exclusive. One of the two is required.

        :param str class_id: ID of year group class (e.g MNB-BIOS-2017-HØST-H)
        :param date start: Start of interval (YYYY-MM-DD)
        :param date end: End of interval (YYYY-MM-DD)
        :param str semester: Semester code (e.g. 18h for autumn 2018)
        :param bool split_intervals: Set the split intervals parameter
        """
        self._verify_interval_or_semester(start, end, semester)
        url = self.urls.get_timetable_for_program_yeargroup_class()
        params = {
            "classid[]": class_id,
            "fromdate": start,
            "todate": end,
            "sem": semester,
        }
        if split_intervals is not None:
            params["split_intervals"] = "true" if split_intervals else "false"
        timetable: dict[str, Any] = self.get(url, params=params)
        # we add the class ID to the timetable object
        timetable["requested_class_id"] = class_id
        return self.maybe_object(ClassTimetable, timetable)

    def get_room_data(self, room_id: str) -> RoomData | dict[Any, Any]:
        """
        Get a room.

        :param str room_id: Room ID
        """
        url = self.urls.get_room_data()
        params = {
            "id": room_id,
        }
        room_data: dict[str, Any] = self.get(url, params=params)
        return self.maybe_object(RoomData, room_data)

    def get_all_rooms(self):
        """
        Get all active rooms.
        """
        url = self.urls.get_all_rooms()
        all_rooms: dict[str, Any] = self.get(url)
        return self.maybe_object(Rooms, all_rooms)


def get_client(config_dict: dict[str, Any]):
    """
    Get a TpClient from configuration.
    """
    return TpClient(
        url=config_dict["url"],
        headers=config_dict.get("headers"),
        rewrite_url=config_dict.get("rewrite_url"),
    )
